@@ -4,6 +4,7 @@
 from __future__ import unicode_literals
 import sys
 import json
+import string
 from collections import defaultdict
 
 from .packages import nltk
@@ -17,7 +18,9 @@ from .np_extractors import BaseNPExtractor, FastNPExtractor
 from .taggers import BaseTagger, PatternTagger
 from .tokenizers import BaseTokenizer, WordTokenizer, SentenceTokenizer
 from .sentiments import BaseSentimentAnalyzer, PatternAnalyzer
+from .parsers import BaseParser, PatternParser
 from .translate import Translator
+from .en import suggest
 
 
 class Word(unicode):
@@ -45,18 +48,20 @@ class Word(unicode):
 
     def singularize(self):
         '''Return the singular version of the word as a string.'''
-        return _singularize(self.string)
+        return Word(_singularize(self.string))
 
     def pluralize(self):
         '''Return the plural version of the word as a string.'''
-        return _pluralize(self.string)
+        return Word(_pluralize(self.string))
 
-    def translate(self, from_lang="en", to="en"):
+    def translate(self, from_lang=None, to="en"):
         '''Translate the word to another language using Google's
         Translate API.
 
         .. versionadded:: 0.5.0
         '''
+        if from_lang is None:
+            from_lang = self.translator.detect(self.string)
         return self.translator.translate(self.string,
                                         from_lang=from_lang, to_lang=to)
 
@@ -67,6 +72,25 @@ class Word(unicode):
         '''
         return self.translator.detect(self.string)
 
+    def spellcheck(self):
+        '''Return a list of (word, confidence) tuples of spelling corrections.
+
+        Based on: Peter Norvig, "How to Write a Spelling Corrector"
+        (http://norvig.com/spell-correct.html) as implemented in the pattern
+        library.
+
+        .. versionadded:: 0.6.0
+        '''
+        return suggest(self.string)
+
+    def correct(self):
+        '''Correct the spelling of the word. Returns the word with the highest
+        confidence using the spelling corrector.
+
+        .. versionadded:: 0.6.0
+        '''
+        return Word(self.spellcheck()[0][0])
+
 
 class WordList(list):
 
@@ -102,6 +126,9 @@ class WordList(list):
         # This is included for Python 2.* compatibility
         return self.__class__(self._collection[i:j])
 
+    def __iter__(self):
+        return iter(self._collection)
+
     def count(self, strg, case_sensitive=False, *args, **kwargs):
         """Get the count of a word or phrase `s` within this WordList.
 
@@ -146,6 +173,38 @@ class WordList(list):
         '''Return the plural version of each word in this WordList.'''
         return [word.pluralize() for word in self]
 
+def _validated_param(obj, name, base_class, default, base_class_name=None):
+    '''Validates a parameter passed to __init__. Makes sure that obj is
+    the correct class. Return obj if it's not None or falls back to default
+
+    :param obj: The object passed in.
+    :param name: The name of the parameter.
+    :param base_class: The class that obj must inherit from.
+    :param default: The default object to fall back upon if obj is None.
+    '''
+    base_class_name = base_class_name if base_class_name else base_class.__name__
+    if obj is not None and not isinstance(obj, base_class):
+        raise ValueError("{name} must be an instance of {cls}"
+                            .format(name=name, cls=base_class_name))
+    return obj if obj else default
+
+def _initialize_models(obj, tokenizer, pos_tagger,
+                            np_extractor, analyzer, parser, classifier):
+    """Common initialization between BaseBlob and Blobber classes."""
+    # tokenizer may be a textblob or an NLTK tokenizer
+    obj.tokenizer = _validated_param(tokenizer, "tokenizer",
+                                    base_class=(BaseTokenizer, nltk.tokenize.api.TokenizerI),
+                                    default=BaseBlob.tokenizer,
+                                    base_class_name="BaseTokenizer")
+    obj.np_extractor = _validated_param(np_extractor, "np_extractor",
+                                        base_class=BaseNPExtractor,
+                                        default=BaseBlob.np_extractor)
+    obj.pos_tagger = _validated_param(pos_tagger, "pos_tagger",
+                                        BaseTagger, BaseBlob.pos_tagger)
+    obj.analyzer = _validated_param(analyzer, "analyzer",
+                                     BaseSentimentAnalyzer, BaseBlob.analyzer)
+    obj.parser = _validated_param(parser, "parser", BaseParser, BaseBlob.parser)
+    obj.classifier = classifier
 
 @python_2_unicode_compatible
 class BaseBlob(ComparableMixin):
@@ -155,11 +214,20 @@ class BaseBlob(ComparableMixin):
     basic dunder and string methods for making objects like Python strings.
 
     :param text: A string.
-    :param tokenizer: (optional) A tokenizer instance. If ``None``, defaults to :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
-    :param np_extractor: (optional) An NPExtractor instance. If ``None``, defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
-    :param pos_tagger: (optional) A Tagger instance. If ``None``, defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
-    :param analyzer: (optional) A sentiment analyzer. If ``None``, defaults to :class:`PatternAnalyzer` <text.sentiments.PatternAnalyzer>`.
-    :param clean_html: (optional) Remove HTML markup from ``text``.
+    :param tokenizer: (optional) A tokenizer instance. If ``None``,
+        defaults to :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
+    :param np_extractor: (optional) An NPExtractor instance. If ``None``,
+        defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+    :param pos_tagger: (optional) A Tagger instance. If ``None``,
+        defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
+    :param analyzer: (optional) A sentiment analyzer. If ``None``,
+        defaults to :class:`PatternAnalyzer <text.sentiments.PatternAnalyzer>`.
+    :param parser: A parser. If ``None``, defaults to
+        :class:`PatternParser <text.parsers.PatternParser>`.
+    :param classifier: A classifier.
+
+    .. versionchanged:: 0.6.0
+        ``clean_html`` parameter deprecated, as it was in NLTK.
     '''
 
     np_extractor = FastNPExtractor()
@@ -167,33 +235,21 @@ class BaseBlob(ComparableMixin):
     tokenizer = WordTokenizer()
     translator = Translator()
     analyzer = PatternAnalyzer()
+    parser = PatternParser()
 
     def __init__(self, text, tokenizer=None,
                 pos_tagger=None, np_extractor=None, analyzer=None,
-                clean_html=False):
+                parser=None, classifier=None, clean_html=False):
         if type(text) not in string_types:
             raise TypeError('The `text` argument passed to `__init__(text)` '
                             'must be a string, not {0}'.format(type(text)))
-        self.raw = self.string = text if not clean_html else nltk.clean_html(text)
+        if clean_html:
+            raise NotImplementedError("clean_html has been deprecated. "
+                                    "To remove HTML markup, use BeautifulSoup's "
+                                    "get_text() function")
+        self.raw = self.string = text
         self.stripped = lowerstrip(self.raw, all=True)
-        # tokenizer may be a textblob or an NLTK tokenizer
-        if (tokenizer is not None and
-                not (isinstance(tokenizer, BaseTokenizer) or
-                    isinstance(tokenizer, nltk.tokenize.api.TokenizerI))):
-            raise ValueError("tokenizer must be an instance of BaseTokenizer")
-        self.tokenizer = tokenizer if tokenizer else BaseBlob.tokenizer
-        if (np_extractor is not None and
-                not isinstance(np_extractor, BaseNPExtractor)):
-            raise ValueError("np_extractor must be an instance of BaseNPExtractor")
-        self.np_extractor = np_extractor if np_extractor else BaseBlob.np_extractor
-        if (pos_tagger is not None and
-                not isinstance(pos_tagger, BaseTagger)):
-            raise ValueError("pos_tagger must be an instance of BaseTagger")
-        self.pos_tagger = pos_tagger if pos_tagger else BaseBlob.pos_tagger
-        if (analyzer is not None and
-                not isinstance(analyzer, BaseSentimentAnalyzer)):
-            raise ValueError("analyzer must be an instance of BaseSentimentAnalyzer")
-        self.analyzer = analyzer if analyzer else BaseBlob.analyzer
+        _initialize_models(self, tokenizer, pos_tagger, np_extractor, analyzer, parser, classifier)
 
     @cached_property
     def words(self):
@@ -201,7 +257,7 @@ class BaseBlob(ComparableMixin):
         If you want to include punctuation characters, access the ``tokens``
         property.
         '''
-        return WordList(WordTokenizer().tokenize(self.raw, include_punc=False))
+        return WordList(WordTokenizer().itokenize(self.raw, include_punc=False))
 
     @cached_property
     def tokens(self):
@@ -218,6 +274,21 @@ class BaseBlob(ComparableMixin):
         t = tokenizer if tokenizer is not None else self.tokenizer
         return WordList(t.tokenize(self.raw))
 
+    def parse(self, parser=None):
+        '''Parse the text.
+
+        :param parser: (optional) A parser instance. If ``None``, defaults to this blob's default parser.
+
+        .. versionadded:: 0.6.0
+        '''
+        p = parser if parser is not None else self.parser
+        return p.parse(self.raw)
+
+    def classify(self):
+        if self.classifier is None:
+            raise NameError("This blob has no classfier. Train one first!")
+        return self.classifier.classify(self.raw)
+
     @cached_property
     def sentiment(self):
         '''Return a tuple of form (polarity, subjectivity ) where polarity
@@ -300,7 +371,7 @@ class BaseBlob(ComparableMixin):
                             for i in range(len(self.words) - n + 1)]
         return grams
 
-    def translate(self, from_lang="en", to="en"):
+    def translate(self, from_lang=None, to="en"):
         '''Translate the blob to another language.
         Uses the Google Translate API. Returns a new TextBlob.
 
@@ -318,11 +389,14 @@ class BaseBlob(ComparableMixin):
 
         .. versionadded:: 0.5.0.
 
-        :param from_lang: Language to translate from.
+        :param from_lang: Language to translate from. If ``None``, will attempt
+            to detect the language.
         :param to: Language to translate to.
-        :rtype: Blob
+        :rtype: BaseBlob
 
         '''
+        if from_lang is None:
+            from_lang = self.translator.detect(self.string)
         return self.__class__(self.translator.translate(self.raw,
                         from_lang=from_lang, to_lang=to))
 
@@ -348,6 +422,25 @@ class BaseBlob(ComparableMixin):
         '''
         return self.translator.detect(self.raw)
 
+    def correct(self):
+        '''Attempt to correct the spelling of a blob.
+
+        .. versionadded:: 0.6.0
+
+        :rtype: BaseBlob
+        '''
+        tok = WordTokenizer()
+        corrected = (Word(w).correct() for w in tok.tokenize(self.raw, include_punc=True))
+        # Separate each token with a space unless the token is a punctuation
+        ret = ''
+        for i, word in enumerate(corrected):
+            # Avoid an extra space at the beginning
+            if word in string.punctuation or i == 0:
+                ret = ''.join([ret, word])
+            else:
+                ret = ' '.join([ret, word])
+        return self.__class__(ret)
+
     def __repr__(self):
         '''Returns a string representation for debugging.'''
         class_name = self.__class__.__name__
@@ -510,13 +603,13 @@ class TextBlob(BaseBlob):
     :param np_extractor: (optional) An NPExtractor instance. If ``None``, defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
     :param pos_tagger: (optional) A Tagger instance. If ``None``, defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
     :param analyzer: (optional) A sentiment analyzer. If ``None``, defaults to :class:`PatternAnalyzer` <text.sentiments.PatternAnalyzer>`.
-    :param clean_html: (optional) Remove HTML markup from ``text``.
+    :param classsifier: (optional) A classifier.
     """
 
     @cached_property
     def sentences(self):
         '''Return list of :class:`Sentence <Sentence>` objects.'''
-        return TextBlob._create_sentence_objects(self.raw)
+        return self._create_sentence_objects()
 
     @cached_property
     def words(self):
@@ -531,7 +624,6 @@ class TextBlob(BaseBlob):
             words.extend(WordTokenizer().tokenize(sent.raw, include_punc=False))
         return WordList(words)
 
-
     @property
     def raw_sentences(self):
         '''List of strings, the raw sentences in the blob.'''
@@ -542,7 +634,6 @@ class TextBlob(BaseBlob):
         '''Returns a list of each sentence's dict representation.'''
         return [sentence.dict for sentence in self.sentences]
 
-
     def to_json(self, *args, **kwargs):
         '''Return a json representation (str) of this blob.
         Takes the same arguments as json.dumps.
@@ -561,8 +652,7 @@ class TextBlob(BaseBlob):
         '''
         return self.to_json()
 
-    @staticmethod
-    def _create_sentence_objects(blob):
+    def _create_sentence_objects(self):
         '''Returns a list of Sentence objects given
         a list of sentence strings. Attempts to handle sentences that
         have more than one punctuation mark at the end of the sentence.
@@ -570,17 +660,20 @@ class TextBlob(BaseBlob):
         '''
         sent_tokenizer = SentenceTokenizer()
         sentence_objects = []
-        sentences = sent_tokenizer.tokenize(blob)  # List of raw sentences
+        sentences = sent_tokenizer.itokenize(self.raw)
         char_index = 0  # Keeps track of character index within the blob
         for sent in sentences:
             # Compute the start and end indices of the sentence
             # within the blob
-            start_index = blob.index(sent, char_index)
+            start_index = self.raw.index(sent, char_index)
             char_index += len(sent)
-            # Create a Sentence object and add it the the list
             end_index = start_index + len(sent)
-            sentence_objects.append(Sentence(sent,
-                    start_index=start_index, end_index=end_index))
+            # Sentences share the same models as their parent blob
+            s = Sentence(sent, start_index=start_index, end_index=end_index,
+                tokenizer=self.tokenizer, np_extractor=self.np_extractor,
+                pos_tagger=self.pos_tagger, analyzer=self.analyzer,
+                parser=self.parser, classifier=self.classifier)
+            sentence_objects.append(s)
         return sentence_objects
 
 
@@ -614,10 +707,11 @@ class Sentence(BaseBlob):
             'subjectivity': self.subjectivity,
         }
 
+
 class Blobber(object):
 
     '''A factory for TextBlobs that all share the same tagger,
-    tokenizer, and np_extractor.
+    tokenizer, parser, classifier, and np_extractor.
 
     Usage:
 
@@ -627,40 +721,33 @@ class Blobber(object):
         >>> tb = Blobber(pos_tagger=NLTKTagger(), tokenizer=SentenceTokenizer())
         >>> blob1 = tb("This is one blob.")
         >>> blob2 = tb("This blob has the same tagger and tokenizer.")
-        >>> blob1.pos_tagger == blob2.pos_tagger
+        >>> blob1.pos_tagger is blob2.pos_tagger
         True
 
-    :param tokenizer: A tokenizer. Default: :class:`WordTokenizer <text.tokenizers.WordTokenizer>`.
-    :param pos_tagger: A POS tagger. Default: :class:`PatternTagger <text.taggers.PatternTagger>`.
-    :param np_extractor: A NP extractor. Default: :class:`FastNPExtractor <text.np_extractors.FastNPExtractor>`.
-    :param analyzer: (optional) A sentiment analyzer. If ``None``, defaults to :class:`PatternAnalyzer` <text.sentiments.PatternAnalyzer>`.
+    :param tokenizer: (optional) A tokenizer instance. If ``None``,
+        defaults to :class:`WordTokenizer() <text.tokenizers.WordTokenizer>`.
+    :param np_extractor: (optional) An NPExtractor instance. If ``None``,
+        defaults to :class:`FastNPExtractor() <text.np_extractors.FastNPExtractor>`.
+    :param pos_tagger: (optional) A Tagger instance. If ``None``,
+        defaults to :class:`PatternTagger <text.taggers.PatternTagger>`.
+    :param analyzer: (optional) A sentiment analyzer. If ``None``,
+        defaults to :class:`PatternAnalyzer <text.sentiments.PatternAnalyzer>`.
+    :param parser: A parser. If ``None``, defaults to
+        :class:`PatternParser <text.parsers.PatternParser>`.
+    :param classifier: A classifier.
+
+    .. versionadded:: 0.4.0
     '''
 
     np_extractor = FastNPExtractor()
     pos_tagger = PatternTagger()
     tokenizer = WordTokenizer()
     analyzer = PatternAnalyzer()
+    parser = PatternParser()
 
     def __init__(self, tokenizer=None, pos_tagger=None, np_extractor=None,
-                analyzer=None):
-        # tokenizer may be a textblob or an NLTK tokenizer
-        if (tokenizer is not None and
-                not (isinstance(tokenizer, BaseTokenizer) or
-                    isinstance(tokenizer, nltk.tokenize.api.TokenizerI))):
-            raise ValueError("tokenizer must be an instance of BaseTokenizer")
-        self.tokenizer = tokenizer if tokenizer else Blobber.tokenizer
-        if (np_extractor is not None and
-                not isinstance(np_extractor, BaseNPExtractor)):
-            raise ValueError("np_extractor must be an instance of BaseNPExtractor")
-        self.np_extractor = np_extractor if np_extractor else Blobber.np_extractor
-        if (pos_tagger is not None and
-                not isinstance(pos_tagger, BaseTagger)):
-            raise ValueError("pos_tagger must be an instance of BaseTagger")
-        self.pos_tagger = pos_tagger if pos_tagger else Blobber.pos_tagger
-        if (analyzer is not None and
-                not isinstance(analyzer, BaseSentimentAnalyzer)):
-            raise ValueError("analyzer must be an instance of BaseSentimentAnalyzer")
-        self.analyzer = analyzer if analyzer else BaseBlob.analyzer
+                analyzer=None, parser=None, classifier=None):
+        _initialize_models(self, tokenizer, pos_tagger, np_extractor, analyzer, parser, classifier)
 
     def __call__(self, text):
         '''Return a new TextBlob object with this Blobber's ``np_extractor``,
@@ -669,17 +756,18 @@ class Blobber(object):
         :returns: A new TextBlob.
         '''
         return TextBlob(text, tokenizer=self.tokenizer, pos_tagger=self.pos_tagger,
-            np_extractor=self.np_extractor, analyzer=self.analyzer)
+                        np_extractor=self.np_extractor, analyzer=self.analyzer,
+                        classifier=self.classifier)
 
     def __repr__(self):
+        classifier_name = self.classifier.__class__.__name__ + "()" if self.classifier else "None"
         return ("Blobber(tokenizer={0}(), pos_tagger={1}(), "
-                    "np_extractor={2}(), analyzer={3}())")\
+                    "np_extractor={2}(), analyzer={3}(), parser={4}(), classifier={5})")\
                     .format(self.tokenizer.__class__.__name__,
                             self.pos_tagger.__class__.__name__,
                             self.np_extractor.__class__.__name__,
-                            self.analyzer.__class__.__name__)
+                            self.analyzer.__class__.__name__,
+                            self.parser.__class__.__name__,
+                            classifier_name)
 
     __str__ = __repr__
-
-
-
