@@ -5,6 +5,7 @@ URL: http://www.clips.ua.ac.be/pages/pattern-web
 Licence: BSD
 '''
 from __future__ import unicode_literals
+import string
 from itertools import chain
 import types
 import os
@@ -52,6 +53,13 @@ def encode_string(v, encoding="utf-8"):
 decode_utf8 = decode_string
 encode_utf8 = encode_string
 
+def isnumeric(strg):
+    try:
+        float(strg)
+    except ValueError:
+        return False
+    return True
+
 #--- LAZY DICTIONARY -------------------------------------------------------------------------------
 # A lazy dictionary is empty until one of its methods is called.
 # This way many instances (e.g., lexicons) can be created without using memory until used.
@@ -978,6 +986,108 @@ def find_tags(tokens, lexicon={}, default=("NN", "NNP", "CD"), language="en", ma
         tagged = [list(map(token, tag)) or [token, default[0]] for token, tag in tagged]
     return tagged
 
+#--- PHRASE CHUNKER --------------------------------------------------------------------------------
+
+SEPARATOR = "/"
+
+NN = r"NN|NNS|NNP|NNPS|NNPS?\-[A-Z]{3,4}|PR|PRP|PRP\$"
+VB = r"VB|VBD|VBG|VBN|VBP|VBZ"
+JJ = r"JJ|JJR|JJS"
+RB = r"(?<!W)RB|RBR|RBS"
+
+# Chunking rules.
+# CHUNKS[0] = Germanic: RB + JJ precedes NN ("the round table").
+# CHUNKS[1] = Romance: RB + JJ precedes or follows NN ("la table ronde", "une jolie fille").
+CHUNKS = [[
+    # Germanic languages: en, de, nl, ...
+    (  "NP", re.compile(r"(("+NN+")/)*((DT|CD|CC|CJ)/)*(("+RB+"|"+JJ+")/)*(("+NN+")/)+")),
+    (  "VP", re.compile(r"(((MD|"+RB+")/)*(("+VB+")/)+)+")),
+    (  "VP", re.compile(r"((MD)/)")),
+    (  "PP", re.compile(r"((IN|PP|TO)/)+")),
+    ("ADJP", re.compile(r"((CC|CJ|"+RB+"|"+JJ+")/)*(("+JJ+")/)+")),
+    ("ADVP", re.compile(r"(("+RB+"|WRB)/)+")),
+], [
+    # Romance languages: es, fr, it, ...
+    (  "NP", re.compile(r"(("+NN+")/)*((DT|CD|CC|CJ)/)*(("+RB+"|"+JJ+")/)*(("+NN+")/)+(("+RB+"|"+JJ+")/)*")),
+    (  "VP", re.compile(r"(((MD|"+RB+")/)*(("+VB+")/)+(("+RB+")/)*)+")),
+    (  "VP", re.compile(r"((MD)/)")),
+    (  "PP", re.compile(r"((IN|PP|TO)/)+")),
+    ("ADJP", re.compile(r"((CC|CJ|"+RB+"|"+JJ+")/)*(("+JJ+")/)+")),
+    ("ADVP", re.compile(r"(("+RB+"|WRB)/)+")),
+]]
+
+# Handle ADJP before VP, so that
+# RB prefers next ADJP over previous VP.
+CHUNKS[0].insert(1, CHUNKS[0].pop(3))
+CHUNKS[1].insert(1, CHUNKS[1].pop(3))
+
+def find_chunks(tagged, language="en"):
+    """ The input is a list of [token, tag]-items.
+        The output is a list of [token, tag, chunk]-items:
+        The/DT nice/JJ fish/NN is/VBZ dead/JJ ./. =>
+        The/DT/B-NP nice/JJ/I-NP fish/NN/I-NP is/VBZ/B-VP dead/JJ/B-ADJP ././O
+    """
+    chunked = [x for x in tagged]
+    tags = "".join("%s%s" % (tag, SEPARATOR) for token, tag in tagged)
+    # Use Germanic or Romance chunking rules according to given language.
+    for tag, rule in CHUNKS[int(language in ("ca", "es", "pt", "fr", "it", "pt", "ro"))]:
+        for m in rule.finditer(tags):
+            # Find the start of chunks inside the tags-string.
+            # Number of preceding separators = number of preceding tokens.
+            i = m.start()
+            j = tags[:i].count(SEPARATOR)
+            n = m.group(0).count(SEPARATOR)
+            for k in range(j, j+n):
+                if len(chunked[k]) == 3:
+                    continue
+                if len(chunked[k]) < 3:
+                    # A conjunction can not be start of a chunk.
+                    if k == j and chunked[k][1] in ("CC", "CJ", "KON", "Conj(neven)"):
+                        j += 1
+                    # Mark first token in chunk with B-.
+                    elif k == j:
+                        chunked[k].append("B-"+tag)
+                    # Mark other tokens in chunk with I-.
+                    else:
+                        chunked[k].append("I-"+tag)
+    # Mark chinks (tokens outside of a chunk) with O-.
+    for chink in filter(lambda x: len(x) < 3, chunked):
+        chink.append("O")
+    # Post-processing corrections.
+    for i, (word, tag, chunk) in enumerate(chunked):
+        if tag.startswith("RB") and chunk == "B-NP":
+            # "Very nice work" (NP) <=> "Perhaps" (ADVP) + "you" (NP).
+            if i < len(chunked)-1 and not chunked[i+1][1].startswith("JJ"):
+                chunked[i+0][2] = "B-ADVP"
+                chunked[i+1][2] = "B-NP"
+    return chunked
+
+def find_prepositions(chunked):
+    """ The input is a list of [token, tag, chunk]-items.
+        The output is a list of [token, tag, chunk, preposition]-items.
+        PP-chunks followed by NP-chunks make up a PNP-chunk.
+    """
+    # Tokens that are not part of a preposition just get the O-tag.
+    for ch in chunked:
+        ch.append("O")
+    for i, chunk in enumerate(chunked):
+        if chunk[2].endswith("PP") and chunk[-1] == "O":
+            # Find PP followed by other PP, NP with nouns and pronouns, VP with a gerund.
+            if i < len(chunked)-1 and \
+             (chunked[i+1][2].endswith(("NP", "PP")) or \
+              chunked[i+1][1] in ("VBG", "VBN")):
+                chunk[-1] = "B-PNP"
+                pp = True
+                for ch in chunked[i+1:]:
+                    if not (ch[2].endswith(("NP", "PP")) or ch[1] in ("VBG", "VBN")):
+                        break
+                    if ch[2].endswith("PP") and pp:
+                        ch[-1] = "I-PNP"
+                    if not ch[2].endswith("PP"):
+                        ch[-1] = "I-PNP"
+                        pp = False
+    return chunked
+
 #### PARSER ########################################################################################
 
 #--- PARSER ----------------------------------------------------------------------------------------
@@ -1179,3 +1289,89 @@ class TaggedString(str):
             for token in sentence.split(" ")]
                 for sentence in str.split(self, "\n")]
 
+#### SPELLING CORRECTION ###########################################################################
+# Based on: Peter Norvig, "How to Write a Spelling Corrector", http://norvig.com/spell-correct.html
+
+class Spelling(lazydict):
+
+    ALPHA = "abcdefghijklmnopqrstuvwxyz"
+
+    def __init__(self, path=""):
+        self._path = path
+
+    def load(self):
+        for x in _read(self._path):
+            x = x.split()
+            dict.__setitem__(self, x[0], int(x[1]))
+
+    @property
+    def path(self):
+        return self._path
+
+    @property
+    def language(self):
+        return self._language
+
+    @classmethod
+    def train(self, s, path="spelling.txt"):
+        """ Counts the words in the given string and saves the probabilities at the given path.
+            This can be used to generate a new model for the Spelling() constructor.
+        """
+        model = {}
+        for w in re.findall("[a-z]+", s.lower()):
+            model[w] = w in model and model[w] + 1 or 1
+        model = ("%s %s" % (k, v) for k, v in sorted(model.items()))
+        model = "\n".join(model)
+        f = open(path, "w")
+        f.write(model)
+        f.close()
+
+    def _edit1(self, w):
+        """ Returns a set of words with edit distance 1 from the given word.
+        """
+        # Of all spelling errors, 80% is covered by edit distance 1.
+        # Edit distance 1 = one character deleted, swapped, replaced or inserted.
+        split = [(w[:i], w[i:]) for i in range(len(w) + 1)]
+        delete, transpose, replace, insert = (
+            [a + b[1:] for a, b in split if b],
+            [a + b[1] + b[0] + b[2:] for a, b in split if len(b) > 1],
+            [a + c + b[1:] for a, b in split for c in Spelling.ALPHA if b],
+            [a + c + b[0:] for a, b in split for c in Spelling.ALPHA]
+        )
+        return set(delete + transpose + replace + insert)
+
+    def _edit2(self, w):
+        """ Returns a set of words with edit distance 2 from the given word
+        """
+        # Of all spelling errors, 99% is covered by edit distance 2.
+        # Only keep candidates that are actually known words (20% speedup).
+        return set(e2 for e1 in self._edit1(w) for e2 in self._edit1(e1) if e2 in self)
+
+    def _known(self, words=[]):
+        """ Returns the given list of words filtered by known words.
+        """
+        return set(w for w in words if w in self)
+
+    def suggest(self, w):
+        """ Return a list of (word, confidence) spelling corrections for the given word,
+            based on the probability of known words with edit distance 1-2 from the given word.
+        """
+        # Don't correct punctuation or numbers
+        if any([w in string.punctuation,
+                isnumeric(w)]):
+            return [(w, 1.0)]
+        if len(self) == 0:
+            self.load()
+        candidates = self._known([w]) \
+                  or self._known(self._edit1(w)) \
+                  or self._known(self._edit2(w)) \
+                  or [w]
+        candidates = [(self.get(c, 0.0), c) for c in candidates]
+        s = float(sum(p for p, word in candidates) or 1)
+        candidates = sorted(((p / s, word) for p, word in candidates), reverse=True)
+        if w.istitle():  # Preserve capitalization
+            candidates = [(word.title(), p) for p, word in candidates]
+        else:
+            candidates = [(word, p) for p, word in candidates]
+        return candidates
+
